@@ -1,6 +1,6 @@
 import numpy as np
 from itertools import combinations
-from pyscf import gto, scf, symm, tools, ao2mo
+from pyscf import gto, scf, symm, ao2mo
 from openfermion import QubitOperator, FermionOperator, jordan_wigner, utils, linalg
 from qiskit import opflow, quantum_info
 from qiskit_nature.operators.second_quantization import FermionicOp
@@ -704,7 +704,7 @@ def get_molecule_name(mol):
         string = 'NH₃'
     return string
 
-def QubitOperator_to_PauliSumOp(qubitoperator):
+def QubitOperator_to_PauliSumOp(qubitoperator, num_qubits = None):
     """Converts a qubit operator stored as in an OpenFermion format (an openfermion.QubitOperator object) into the equivalent operator in Qiskit format (a qiskit.opflow.PauliSumOp object)
 
     Args:
@@ -713,7 +713,12 @@ def QubitOperator_to_PauliSumOp(qubitoperator):
     Returns:
         opflow.PauliSumOp: the qiskit.opflow.PauliSumOp output
     """
-    N = utils.count_qubits(qubitoperator)
+    if num_qubits == None:
+        N = utils.count_qubits(qubitoperator)
+    else:
+        N = num_qubits
+    if N == 0:
+        return 0
     output = 0*quantum_info.SparsePauliOp('I'*N)
     for key in qubitoperator.terms:
         if not key:
@@ -731,19 +736,21 @@ def QubitOperator_to_PauliSumOp(qubitoperator):
                         count += 1
                     else:
                         string += 'I'
-        output += qubitoperator.terms[key]*quantum_info.SparsePauliOp(string)
+        output += qubitoperator.terms[key]*quantum_info.SparsePauliOp(string[::-1])
     return opflow.PauliSumOp(output).reduce()
 
 def PauliSumOp_to_QubitOperator(input):
     output = QubitOperator()
     for k in input:
+        coeff = k.to_pauli_op().coeff
+        coeffs = k.coeffs
         k = str(k.to_pauli_op())
         N = len(k)
         term = str()
         for i, s in enumerate(k):
             if s == 'X' or s == 'Z' or s == 'Y':
-                term += s + str(N - i) + ' '
-        output += QubitOperator(term)
+                term += s + str(N - i - 1) + ' '
+        output += coeff*QubitOperator(term)
     return output
 
 def find_ground_state_irrep(orbital_labels, mo_occ, character_table, irrep_labels):
@@ -835,9 +842,12 @@ def make_encoding(atom, basis, charge = 0, spin = 0, irrep = None):
     mol.spin = spin
     mol.verbose = 0
     mol.build()
-    if mol.groupname == 'Dooh' or mol.groupname == 'Coov' or mol.groupname == 'SO3':
+    if mol.groupname == 'Dooh' or mol.groupname == 'SO3':
         mol.symmetry = 'D2h'
         mol.build()
+    if mol.groupname == 'Coov':
+        mol.symmetry = 'C2v'
+        mol.build()
     mf = scf.RHF(mol)
     mf.kernel()
     
@@ -873,7 +883,7 @@ def apply_encoding(operator, encoding, output_format = 'openfermion'):
     if output_format == 'openfermion':
         return reduced_operator
     elif output_format == 'qiskit':
-        reduced_operator = QubitOperator_to_PauliSumOp(reduced_operator)
+        reduced_operator = QubitOperator_to_PauliSumOp(reduced_operator, num_qubits= len(tableau)//2 - len(target_qubits))
         return reduced_operator
 
 def reduced_hamiltonian(atom, basis, charge = 0, spin = 0, irrep = None, verbose = True, show_lowest_eigenvalue = False, output_format = 'openfermion'):
@@ -979,13 +989,13 @@ def reduced_hamiltonian(atom, basis, charge = 0, spin = 0, irrep = None, verbose
         hamiltonian_table1 +
         '<p><i>(' + f'{length1:,}' + ' terms on ' + str(qubits1) + ' qubits)</i></p>')
         if show_lowest_eigenvalue == True:
-            to_print += '<p>Its lowest eigenvalue is ' + f'{ground_state1[0]:.5f}' + ' Ha.</p>'
+            to_print += '<p>Its lowest eigenvalue is ' + f'{ground_state1[0]:.15f}' + ' Ha.</p>'
         to_print += ('<h2>Qubit-reduced Hamiltonian</h2>' +
         '<p>The <b>qubit-reduced Hamiltonian</b> in the symmetry-adapted encoding (obtained by changing basis by applying the Clifford tableau above, eliminating the target qubits ' + str(target_qubits)[1:-1] + ' and relabelling the remaining qubits) is:</p>' +
         hamiltonian_table3 +
         '<p><i>(' + f'{length3:,}' + ' terms on ' + str(qubits3) + ' qubits)</i></p>')
         if show_lowest_eigenvalue == True:
-            to_print += '<p>Its lowest eigenvalue is ' + f'{ground_state3[0]:.5f}' + ' Ha.</p>'
+            to_print += '<p>Its lowest eigenvalue is ' + f'{ground_state3[0]:.15f}' + ' Ha.</p>'
 
         display(HTML(to_print))
     
